@@ -105,7 +105,7 @@ class FirewallIPSet(object):
                     except Exception as msg:
                         raise FirewallError(errors.COMMAND_FAILED, msg)
 
-            if self._fw.individual_calls():
+            if self._fw._individual_calls:
                 try:
                     backend.set_create(obj.name, obj.type, obj.options)
                 except Exception as msg:
@@ -160,8 +160,8 @@ class FirewallIPSet(object):
 
     # TYPE
 
-    def get_type(self, name):
-        return self.get_ipset(name, applied=True).type
+    def get_type(self, name, applied=True):
+        return self.get_ipset(name, applied=applied).type
 
     # DIMENSION
     def get_dimension(self, name):
@@ -178,8 +178,8 @@ class FirewallIPSet(object):
 
     # OPTIONS
 
-    def get_family(self, name):
-        obj = self.get_ipset(name, applied=True)
+    def get_family(self, name, applied=True):
+        obj = self.get_ipset(name, applied=applied)
         if "family" in obj.options:
             if obj.options["family"] == "inet6":
                 return "ipv6"
@@ -255,7 +255,7 @@ class FirewallIPSet(object):
 
         try:
             for backend in self.backends():
-                if self._fw.individual_calls():
+                if self._fw._individual_calls:
                     for entry in obj.entries:
                         backend.set_add(obj.name, entry)
                 else:
