@@ -38,9 +38,9 @@ from firewall.server.decorators import handle_exceptions, \
 from firewall.server.config_icmptype import FirewallDConfigIcmpType
 from firewall.server.config_service import FirewallDConfigService
 from firewall.server.config_zone import FirewallDConfigZone
+from firewall.server.config_policy import FirewallDConfigPolicy
 from firewall.server.config_ipset import FirewallDConfigIPSet
 from firewall.server.config_helper import FirewallDConfigHelper
-from firewall.core.io.zone import Zone
 from firewall.core.io.icmptype import IcmpType
 from firewall.core.io.ipset import IPSet
 from firewall.core.io.helper import Helper
@@ -85,6 +85,8 @@ class FirewallDConfig(slip.dbus.service.Object):
         self.watcher.add_watch_dir(config.ETC_FIREWALLD_SERVICES)
         self.watcher.add_watch_dir(config.FIREWALLD_ZONES)
         self.watcher.add_watch_dir(config.ETC_FIREWALLD_ZONES)
+        self.watcher.add_watch_dir(config.FIREWALLD_POLICIES)
+        self.watcher.add_watch_dir(config.ETC_FIREWALLD_POLICIES)
         # Add watches for combined zone directories
         if os.path.exists(config.ETC_FIREWALLD_ZONES):
             for filename in sorted(os.listdir(config.ETC_FIREWALLD_ZONES)):
@@ -107,6 +109,7 @@ class FirewallDConfig(slip.dbus.service.Object):
                                                 "FirewallBackend": "readwrite",
                                                 "FlushAllOnReload": "readwrite",
                                                 "RFC3964_IPv4": "readwrite",
+                                                "AllowZoneDrifting": "readwrite",
                                               })
 
     @handle_exceptions
@@ -121,6 +124,8 @@ class FirewallDConfig(slip.dbus.service.Object):
         self.zone_idx = 0
         self.helpers = [ ]
         self.helper_idx = 0
+        self.policy_objects = [ ]
+        self.policy_object_idx = 0
 
         for ipset in self.config.get_ipsets():
             self._addIPSet(self.config.get_ipset(ipset))
@@ -132,6 +137,8 @@ class FirewallDConfig(slip.dbus.service.Object):
             self._addZone(self.config.get_zone(zone))
         for helper in self.config.get_helpers():
             self._addHelper(self.config.get_helper(helper))
+        for policy in self.config.get_policy_objects():
+            self._addPolicy(self.config.get_policy_object(policy))
 
     @handle_exceptions
     def __del__(self):
@@ -159,6 +166,10 @@ class FirewallDConfig(slip.dbus.service.Object):
             item = self.helpers.pop()
             item.unregister()
             del item
+        while len(self.policy_objects) > 0:
+            item = self.policy_objects.pop()
+            item.unregister()
+            del item
         self._init_vars()
 
     @handle_exceptions
@@ -291,6 +302,20 @@ class FirewallDConfig(slip.dbus.service.Object):
                 return
             self.Updated()
 
+        elif (name.startswith(config.FIREWALLD_POLICIES) or \
+              name.startswith(config.ETC_FIREWALLD_POLICIES)) and \
+             name.endswith(".xml"):
+            try:
+                (what, obj) = self.config.update_policy_object_from_path(name)
+            except Exception as msg:
+                log.error("Failed to load policy file '%s': %s" % (name, msg))
+                return
+            if what == "new":
+                self._addPolicy(obj)
+            elif what == "remove":
+                self.removePolicy(obj)
+            elif what == "update":
+                self._updatePolicy(obj)
 
     @handle_exceptions
     def _addIcmpType(self, obj):
@@ -324,6 +349,14 @@ class FirewallDConfig(slip.dbus.service.Object):
                 zone.obj = self.config.set_zone_config(zone.obj, settings)
                 zone.Updated(zone.obj.name)
 
+        for policy in self.policy_objects:
+            settings = policy.getSettings()
+            # if this IcmpType is used in a policy remove it from that policy first
+            if "icmp_blocks" in settings and obj.name in settings["icmp_blocks"]:
+                settings["icmp_blocks"].remove(obj.name)
+                policy.obj = self.config.set_policy_object_config_dict(policy.obj, settings)
+                policy.Updated(policy.obj.name)
+
         for icmptype in self.icmptypes:
             if icmptype.obj == obj:
                 icmptype.Removed(obj.name)
@@ -362,6 +395,14 @@ class FirewallDConfig(slip.dbus.service.Object):
                 zone.obj = self.config.set_zone_config(zone.obj, settings)
                 zone.Updated(zone.obj.name)
 
+        for policy in self.policy_objects:
+            settings = policy.getSettings()
+            # if this Service is used in a policy remove it from that policy first
+            if "services" in settings and obj.name in settings["services"]:
+                settings["services"].remove(obj.name)
+                policy.obj = self.config.set_policy_object_config_dict(policy.obj, settings)
+                policy.Updated(policy.obj.name)
+
         for service in self.services:
             if service.obj == obj:
                 service.Removed(obj.name)
@@ -397,6 +438,34 @@ class FirewallDConfig(slip.dbus.service.Object):
                 self.zones.remove(zone)
                 del zone
 
+    @handle_exceptions
+    def _addPolicy(self, obj):
+        # TODO: check for idx overflow
+        config_policy = FirewallDConfigPolicy(
+            self, self.config, obj, self.policy_object_idx, self.busname,
+            "%s/%d" % (config.dbus.DBUS_PATH_CONFIG_POLICY, self.policy_object_idx))
+        self.policy_objects.append(config_policy)
+        self.policy_object_idx += 1
+        self.PolicyAdded(obj.name)
+        return config_policy
+
+    @handle_exceptions
+    def _updatePolicy(self, obj):
+        for policy in self.policy_objects:
+            if policy.obj.name == obj.name and policy.obj.path == obj.path and \
+                    policy.obj.filename == obj.filename:
+                policy.obj = obj
+                policy.Updated(obj.name)
+
+    @handle_exceptions
+    def removePolicy(self, obj):
+        for policy in self.policy_objects:
+            if policy.obj == obj:
+                policy.Removed(obj.name)
+                policy.unregister()
+                self.policy_objects.remove(policy)
+                del policy
+
     @handle_exceptions
     def _addIPSet(self, obj):
         # TODO: check for idx overflow
@@ -487,7 +556,8 @@ class FirewallDConfig(slip.dbus.service.Object):
         if prop not in [ "DefaultZone", "MinimalMark", "CleanupOnExit",
                          "Lockdown", "IPv6_rpfilter", "IndividualCalls",
                          "LogDenied", "AutomaticHelpers", "FirewallBackend",
-                         "FlushAllOnReload", "RFC3964_IPv4" ]:
+                         "FlushAllOnReload", "RFC3964_IPv4",
+                         "AllowZoneDrifting" ]:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.InvalidArgs: "
                 "Property '%s' does not exist" % prop)
@@ -540,6 +610,10 @@ class FirewallDConfig(slip.dbus.service.Object):
             if value is None:
                 value = "yes" if config.FALLBACK_RFC3964_IPV4 else "no"
             return dbus.String(value)
+        elif prop == "AllowZoneDrifting":
+            if value is None:
+                value = "yes" if config.FALLBACK_ALLOW_ZONE_DRIFTING else "no"
+            return dbus.String(value)
 
     @dbus_handle_exceptions
     def _get_dbus_property(self, prop):
@@ -565,6 +639,8 @@ class FirewallDConfig(slip.dbus.service.Object):
             return dbus.String(self._get_property(prop))
         elif prop == "RFC3964_IPv4":
             return dbus.String(self._get_property(prop))
+        elif prop == "AllowZoneDrifting":
+            return dbus.String(self._get_property(prop))
         else:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.InvalidArgs: "
@@ -605,7 +681,8 @@ class FirewallDConfig(slip.dbus.service.Object):
             for x in [ "DefaultZone", "MinimalMark", "CleanupOnExit",
                        "Lockdown", "IPv6_rpfilter", "IndividualCalls",
                        "LogDenied", "AutomaticHelpers", "FirewallBackend",
-                       "FlushAllOnReload", "RFC3964_IPv4" ]:
+                       "FlushAllOnReload", "RFC3964_IPv4",
+                       "AllowZoneDrifting" ]:
                 ret[x] = self._get_property(x)
         elif interface_name in [ config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                                  config.dbus.DBUS_INTERFACE_CONFIG_POLICIES ]:
@@ -633,7 +710,7 @@ class FirewallDConfig(slip.dbus.service.Object):
                                   "IPv6_rpfilter", "IndividualCalls",
                                   "LogDenied", "AutomaticHelpers",
                                   "FirewallBackend", "FlushAllOnReload",
-                                  "RFC3964_IPv4" ]:
+                                  "RFC3964_IPv4", "AllowZoneDrifting" ]:
                 if property_name == "MinimalMark":
                     try:
                         int(new_value)
@@ -677,6 +754,11 @@ class FirewallDConfig(slip.dbus.service.Object):
                         raise FirewallError(errors.INVALID_VALUE,
                                             "'%s' for %s" % \
                                             (new_value, property_name))
+                if property_name == "AllowZoneDrifting":
+                    if new_value.lower() not in ["yes", "true", "no", "false"]:
+                        raise FirewallError(errors.INVALID_VALUE,
+                                            "'%s' for %s" % \
+                                            (new_value, property_name))
 
                 self.config.get_firewalld_conf().set(property_name, new_value)
                 self.config.get_firewalld_conf().write()
@@ -1173,7 +1255,7 @@ class FirewallDConfig(slip.dbus.service.Object):
         return ret[0] if ret else ""
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG,
-                         in_signature='s'+Zone.DBUS_SIGNATURE,
+                         in_signature="s(sssbsasa(ss)asba(ssss)asasasasa(ss)b)",
                          out_signature='o')
     @dbus_handle_exceptions
     def addZone(self, zone, settings, sender=None):
@@ -1192,11 +1274,82 @@ class FirewallDConfig(slip.dbus.service.Object):
         config_zone = self._addZone(obj)
         return config_zone
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG,
+                         in_signature="sa{sv}",
+                         out_signature='o')
+    @dbus_handle_exceptions
+    def addZone2(self, zone, settings, sender=None):
+        """add zone with given name and settings
+        """
+        zone = dbus_to_python(zone, str)
+        settings = dbus_to_python(settings)
+        log.debug1("config.addZone('%s')", zone)
+        self.accessCheck(sender)
+        if "target" in settings and settings["target"] == "default":
+            settings["target"] = DEFAULT_ZONE_TARGET
+        obj = self.config.new_zone_dict(zone, settings)
+        config_zone = self._addZone(obj)
+        return config_zone
+
     @dbus.service.signal(config.dbus.DBUS_INTERFACE_CONFIG, signature='s')
     @dbus_handle_exceptions
     def ZoneAdded(self, zone):
         log.debug1("config.ZoneAdded('%s')" % (zone))
 
+    # policies
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, out_signature='ao')
+    @dbus_handle_exceptions
+    def listPolicies(self, sender=None):
+        """list policies objects paths
+        """
+        log.debug1("config.listPolicies()")
+        return self.policy_objects
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, out_signature='as')
+    @dbus_handle_exceptions
+    def getPolicyNames(self, sender=None):
+        """get policy names
+        """
+        log.debug1("config.getPolicyNames()")
+        policies = [ ]
+        for obj in self.policy_objects:
+            policies.append(obj.obj.name)
+        return sorted(policies)
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG, in_signature='s',
+                         out_signature='o')
+    @dbus_handle_exceptions
+    def getPolicyByName(self, policy, sender=None):
+        """object path of policy with given name
+        """
+        policy = dbus_to_python(policy, str)
+        log.debug1("config.getPolicyByName('%s')", policy)
+        for obj in self.policy_objects:
+            if obj.obj.name == policy:
+                return obj
+        raise FirewallError(errors.INVALID_POLICY, policy)
+
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG,
+                         in_signature="sa{sv}",
+                         out_signature='o')
+    @dbus_handle_exceptions
+    def addPolicy(self, policy, settings, sender=None):
+        """add policy with given name and settings
+        """
+        policy = dbus_to_python(policy, str)
+        settings = dbus_to_python(settings)
+        log.debug1("config.addPolicy('%s')", policy)
+        self.accessCheck(sender)
+        obj = self.config.new_policy_object_dict(policy, settings)
+        config_policy = self._addPolicy(obj)
+        return config_policy
+
+    @dbus.service.signal(config.dbus.DBUS_INTERFACE_CONFIG, signature='s')
+    @dbus_handle_exceptions
+    def PolicyAdded(self, policy):
+        log.debug1("config.PolicyAdded('%s')" % (policy))
+
     # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 
     # H E L P E R S
@@ -1422,7 +1575,9 @@ class FirewallDConfig(slip.dbus.service.Object):
                    (ipv, table, chain, ))
         self.accessCheck(sender)
         settings = list(self.getSettings())
-        settings[1] = []
+        for rule in settings[1][:]:
+            if (ipv, table, chain) == (rule[0], rule[1], rule[2]):
+                settings[1].remove(rule)
         self.update(tuple(settings))
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
